@@ -1,12 +1,11 @@
 import functools
-import json
 import logging
 import sys
 from os import getcwd, path
 from pathlib import Path
 from pprint import pformat
 from subprocess import check_output
-from urllib.parse import urljoin
+from urllib.parse import urljoin, urlparse
 
 import click
 import fsspec
@@ -14,7 +13,7 @@ import requests
 import yaml
 from git import Repo
 
-from nomnomdata.auth import KeyAuth
+from nomnomdata.auth import DEFAULT_PROFILE, NNDAuth, get_profiles
 
 from .model_validator import validate_model
 
@@ -148,12 +147,6 @@ def include_includes(parameters, git_ref, model_fp, remote=None):
     is_flag=True,
     help="Use model files from current working directory instead of git",
 )
-@click.option(
-    "-ns",
-    "--nomitall-secret",
-    required=True,
-    help="Secret key to authenticate with nomitall",
-)
 @click.option(
     "-n",
     "--nomitall",
@@ -174,10 +167,28 @@ def include_includes(parameters, git_ref, model_fp, remote=None):
     default="model.yaml",
     help="Model YAML file to build + deploy",
 )
+@click.option(
+    "-o", "--org", "org", help="UUID of the organization to publish the model as",
+)
 def model_update(
-    work_dir=None, nomitall_secret=None, nomitall=None, dry_run=None, model_fn=None
+    work_dir=None,
+    nomitall_secret=None,
+    nomitall=None,
+    dry_run=None,
+    model_fn=None,
+    org=None,
 ):
     """Update staging or prod nomitall model definitions. Defaults to using files from git master/staging branch for prod/staging"""
+    if nomitall == "nomitall-prod":
+        nomitall_url = "https://user.api.nomnomdata.com/api/1/"
+    elif nomitall == "nomitall-stage":
+        nomitall_url = "https://staging.user.api.nomnomdata.com/api/1/"
+    else:
+        nomitall_url = nomitall
+    if not org:
+        profile = get_profiles()[DEFAULT_PROFILE]
+        netloc = urlparse(nomitall_url).netloc
+        org = profile[netloc]["default-org"]
     relpath = model_fn
     branch = confirm_git_branch(work_dir, nomitall)
     doc = fetch_git_yaml(relpath, branch)
@@ -207,24 +218,15 @@ def model_update(
     else:
         doc["parameters"] = include_includes(doc["parameters"], branch, model_fn)
 
+    if not validate_model(doc):
+        raise click.Abort("Model did not pass validation")
+
     if dry_run:
         _logger.info("PARSED MODEL :")
         _logger.info(pformat(doc))
         sys.exit()
 
-    if not validate_model(doc):
-        sys.exit(1)
-
-    if nomitall == "nomitall-prod":
-        nomitall_url = "https://api.nomnomdata.com/api/1/"
-    elif nomitall == "nomitall-stage":
-        nomitall_url = "https://staging.api.nomnomdata.com/api/1/"
-    else:
-        nomitall_url = nomitall
-
-    update_model(
-        nomitall_url, nomitall_secret, doc, model_type,
-    )
+    update_model(nomitall_url, nomitall_secret, doc, model_type, org)
 
 
 def upload_icons(icons, uuid, session):
@@ -271,11 +273,9 @@ def process_help(doc, key="root"):
     return help_files
 
 
-def update_model(
-    nomitall_url, nomitall_secret, model, model_type,
-):
+def update_model(nomitall_url, nomitall_secret, model, model_type, org):
     session = NomitallSession(prefix_url=nomitall_url)
-    session.auth = KeyAuth(nomitall_secret)
+    session.auth = NNDAuth()
     help_files = {}
     icons = {}
     if model_type == "engine":
@@ -283,7 +283,7 @@ def update_model(
         icons = model.pop("icons", {})
         uuid = model["uuid"]
         _logger.info("Pushing engine to nomitall")
-        resp = session.request("POST", "engine/deploy", json=model)
+        resp = session.request("POST", f"engine/deploy/{org}", json=model)
         check_response(resp)
         upload_help(help_files=help_files, uuid=uuid, session=session)
         upload_icons(icons=icons, uuid=uuid, session=session)
@@ -313,4 +313,7 @@ def check_response(resp):
         if resp.status_code == 401:
             _logger.error(f"Check secret key is valid\n\t\t {resp.json()}")
             raise click.Abort
+        if resp.status_code == 403:
+            _logger.error(f"Check user permissions\n\t\t {resp.json()}")
+            raise click.Abort
     resp.raise_for_status()
